package com.game 
{
	import com.hud.TextWindow;
	import flash.display.Scene;
	import net.flashpunk.graphics.Canvas;
	import net.flashpunk.Sfx;
	import net.flashpunk.Tween;
	import net.flashpunk.Tweener;
	import net.flashpunk.World;
	
	import flash.utils.getQualifiedClassName;
	
	/**
	 * ...
	 * @author ...
	 */
	public class StoryPlayer extends Tweener
	{
		private const NORMAL:int = 0;
		private const NEWSCENE:int = 1;
		private const SHADOW:int = 2;
		
		private var canvas:Canvas;
		private var actions:Array;
		private var textWindow:TextWindow;
		private var currentScene:LDScene;
		private var tweenCounter:Tween;
		
		private var sfx:Sfx;
		private var currAction:Object;
		private var lastAction:Object;
		
		public var worldPtr:World;
		
		
		public function StoryPlayer() 
		{
			currentScene = null;
			actions = new Array();
		}
		
		public function setCanvas(canv:Canvas):void
		{
			canvas = canv;
		}

		public function setTextWindow(text:TextWindow):void
		{
			textWindow = text;
		}		
		
		public function setScene(scene:LDScene):void
		{
			currentScene = scene;
			canvas.drawGraphic(0, 0, currentScene.graphic);
		}
		
		public function play():void
		{
			nextAction();
		}
		
		public function nextAction():void
		{
			if (actions.length == 0)
			{
				tweenCounter = null;
				return;
			}
			
			lastAction = currAction;
			currAction = actions.shift();
			
			if (lastAction != null && lastAction.type == SHADOW)
			{
				worldPtr.remove(lastAction.text);
			}
			
			if (currAction.type == NEWSCENE)
			{
				setScene(currAction.scene);
				nextAction();
			}
			else if (currAction.type == SHADOW)
			{
				if (currAction.anim != "")
					currentScene.actor.play(currAction.anim);
					
				worldPtr.add(currAction.text);
				
				tweenCounter = new Tween(currAction.time, Tween.ONESHOT, currAction.callBack == null ? nextAction : currAction.callBack);
				this.addTween(tweenCounter, true);
				
				if (currAction.sound != null)
				{
					sfx = new Sfx(currAction.sound);
					sfx.play(1);
				}
			}
			else
			{
				if (currAction.anim != "")
					currentScene.actor.play(currAction.anim);
				
				textWindow.addTimedText(currAction.text);			
				
				tweenCounter = new Tween(currAction.time, Tween.ONESHOT, currAction.callBack == null ? nextAction : currAction.callBack);
				this.addTween(tweenCounter, true);
				
				if (currAction.sound != null)
				{
					sfx = new Sfx(currAction.sound);
					sfx.play(1);
				}
			}
		}
		
		
		override public function update():void
		{
			//trace(tweenCounter.percent);
			canvas.drawGraphic(0, 0, currentScene.graphic);
			
			if (tweenCounter.active)
				tweenCounter.update();
				
			super.update();
		}
		
		public function queue(anim:String, text:String, time:int, sound:Class = null, callBack:Function = null):void
		{
			actions.push( { type: NORMAL, anim:anim, text:text, time:time, callBack:callBack, sound:sound } );
		}
		
		public function queueShadowText(anim:String, text:ShadowedText, world:World, time:int, sound:Class = null, callBack:Function = null):void
		{
			actions.push( { type:SHADOW, anim:anim, text:text, world:World, time:time, callBack:callBack, sound:sound } );
		}
		
		public function queueScene(scene:LDScene):void
		{
			actions.push({ type: NEWSCENE, scene: scene});
		}
	}

}